#if defined(_WIN32) || defined(_WIN64)
#include <windows.h>
#include <tchar.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "acquirefile_read.h"

/* (partial) test for acquirefile.dll / libacquirefile.so
   Reads an .acquire file and writes its sampled data to stdout.
   Channels are in columns; segments are separated by an extra newline;
   sweeps are separated by a line of '-'s.

   Windows: use the project in test_acquirefile\ 
*/


#define CHUNK_SIZE (1<<16)

#ifdef UNICODE
  #define ACQUIREFILE_OPEN acquirefile_read_open_w
#else
  #define ACQUIREFILE_OPEN acquirefile_read_open
#endif

#if defined(_WIN32) || defined(_WIN64)
#define sleep Sleep

int test_acquirefile_read(int argc, const _TCHAR** argv)
#else
int main(int argc, char **argv)
#endif
{
  int64_t channel_count, series_count, sweep_count, segment_count, data_count, frames_read;
  int64_t series_i, sweep_i, segment_i, channel_i, offset, chunk_size, i;
  acquirefile_read_series *series;
  acquirefile_read_sweep *sweep;
  acquirefile_read_segment *segment;
  float** channel_data;
  acquirefile_read *file;

  if ( argc != 2 || (argc > 1 && argv[1][0] == '-') ) {
    printf("Usage: ./test_acquirefile_read somefile.acquire\n");
    return 0;
  }
  printf("%s\n", argv[1]);
  
  file = ACQUIREFILE_OPEN(argv[1]);
  if ( ! file )
	  return -1;
  channel_count = file->header.analog_in_channel_count;
  series_count = file->header.series_count;
  
  channel_data = (float**) malloc(sizeof(float*) * channel_count);
  for (channel_i=0; channel_i<channel_count; ++channel_i)
    channel_data[channel_i] = (float*) malloc(sizeof(float) * CHUNK_SIZE);

  for (series_i=0; series_i<series_count; ++series_i) {
    if ( series_i )
      printf("\n--------------------------------------------------------------------------------\n");
    series = file->series + series_i;
    sweep_count = series->descriptor.sweep_count;
    for (sweep_i=0; sweep_i<sweep_count; ++sweep_i) {
      if ( sweep_i )
	printf("\n");
      sweep = series->sweeps[sweep_i];
      segment_count = sweep->descriptor.segment_count;
      for (segment_i=0; segment_i<segment_count; ++segment_i) {
	if ( segment_i )
	  printf("\n");
	segment = & sweep->segments[segment_i];
	data_count = segment->frames;
	for (offset=0; offset<data_count; offset += CHUNK_SIZE) {
	  chunk_size = data_count - offset;
	  if ( chunk_size > CHUNK_SIZE )
	    chunk_size = CHUNK_SIZE;
	  for (channel_i=0; channel_i<channel_count; ++channel_i)
	    frames_read = acquirefile_read_data_float(file, series_i, sweep_i, segment_i, channel_i,
						      offset, chunk_size, channel_data[channel_i]);
	  for (i=0; i<chunk_size; ++i) {
	    for (channel_i=0; channel_i<channel_count; ++channel_i)
	      printf("%s%f", channel_i?"\t":"", channel_data[channel_i][i]);
	    printf("\n");
	  }
	}
      }
    }
  }
  acquirefile_read_close(file);

  sleep(100*1000);

  return 0;
}

